@@ -2,7 +2,7 @@
 
 # 3-Clause BSD License
 # 
-# Copyright (c) 2008-2021, James Philip Rowell,
+# Copyright (c) 2008-2022, James Philip Rowell,
 # Alpha Eleven Incorporated
 # www.alpha-eleven.com
 # All rights reserved.
@@ -61,11 +61,13 @@ import subprocess
 import textwrap
 import math
 import time
+import datetime
+import dateparser
 import copy
 from operator import itemgetter
 import seqLister
 
-VERSION = "2.5.3"     # Semantic Versioning 2.0.0
+VERSION = "2.6.0"     # Semantic Versioning 2.0.0
 
 PROG_NAME = "lsseq"
 
@@ -159,6 +161,21 @@ IMAGE_EXT = [
     "tpic"
 ]
 
+DATE_FORMAT_LIST = [
+    '%y%m%d',
+    '%Y%m%d',
+    '%y%m%d-%H',
+    '%Y%m%d-%H',
+    '%y%m%d-%H%M',
+    '%Y%m%d-%H%M',
+    '%y%m%d-%H%M%S',
+    '%Y%m%d-%H%M%S',
+    '%y%m%d%H%M',    # Undocumented, but kept for compatibility
+    '%Y%m%d%H%M',    # with earlier versions (v2.5.3 and earlier)
+    '%y%m%d%H%M.%S', # of lsseq. Note MMDDhhmm[.ss] is no longers
+    '%Y%m%d%H%M.%S'  # supported, thus not fully backward compatible.
+]
+
 PATH_NOPREFIX = 0
 PATH_ABS = 1
 PATH_REL = 2
@@ -839,7 +856,7 @@ def listSeqDir(dirContents, path, listSubDirs, args, traversedPath) :
 
             if isMovie(k) :
                 # Note: only thing stored in "moviesDictionary" is the mod-time of the file.
-                timeList.append((k, moviesDictionary[k]))
+                timeList.append((k, int(moviesDictionary[k])))
 
             elif isCache(k) :
                 validTimes = []
@@ -862,7 +879,7 @@ def listSeqDir(dirContents, path, listSubDirs, args, traversedPath) :
                             time = (validTimes[midIndex] + validTimes[midIndex-1])/2
                     else : # newest
                         time = validTimes[-1]
-                timeList.append((k, time))
+                timeList.append((k, int(time)))
 
             else : # key is an image.
                 validTimes = []
@@ -885,20 +902,20 @@ def listSeqDir(dirContents, path, listSubDirs, args, traversedPath) :
                             time = (validTimes[midIndex] + validTimes[midIndex-1])/2
                     else : # newest
                         time = validTimes[-1]
-                timeList.append((k, time))
+                timeList.append((k, int(time)))
 
     if args.sortByMTime :
-        timeList.sort(key=itemgetter(MTIME))
+        timeList.sort(key=itemgetter(MTIME)) # Sorts by time.
         # Note: ls -t prints newest first; ls -tr is newest last.
         if not args.reverseListing :
             timeList.reverse()
         for seq in timeList :
             if args.cutoffTime != None :
                 if args.cutoffTime[0] == 'before' :
-                    if seq[MTIME] >= args.cutoffTime[1] :
+                    if seq[MTIME] > args.cutoffTime[1] :
                         continue
                 else : # Guaranteed to be 'since'
-                    if seq[MTIME] <= args.cutoffTime[1] :
+                    if seq[MTIME] < args.cutoffTime[1] :
                         continue
             if isMovie(seq[DICTKEY]) :
                 if args.prependPath != PATH_NOPREFIX :
@@ -914,16 +931,15 @@ def listSeqDir(dirContents, path, listSubDirs, args, traversedPath) :
                 printSeq(seq[DICTKEY], imageDictionary[seq[DICTKEY]], args, traversedPath)
                 somethingWasPrinted = True
     elif args.cutoffTime != None :
-        timeList.sort(key=itemgetter(DICTKEY))
-        # Note: ls -t prints newest first; ls -tr is newest last.
-        if not args.reverseListing :
+        timeList.sort(key=itemgetter(DICTKEY)) # Sorts by name.
+        if args.reverseListing :
             timeList.reverse()
         for seq in timeList :
             if args.cutoffTime[0] == 'before' :
-                if seq[MTIME] >= args.cutoffTime[1] :
+                if seq[MTIME] > args.cutoffTime[1] :
                     continue
             else : # Guaranteed to be 'since'
-                if seq[MTIME] <= args.cutoffTime[1] :
+                if seq[MTIME] < args.cutoffTime[1] :
                     continue
             if isMovie(seq[DICTKEY]) :
                 if args.prependPath != PATH_NOPREFIX :
@@ -1203,8 +1219,11 @@ def main() :
         dest="cutoffTime",
         help="where TENSE is either 'before' or 'since'; only list sequences \
         up to (and including) or after (and including) the time specified. The --time argument \
-        specifies which frame to use for the cutoff comparison",
-        metavar=("TENSE", "[[CC]YY]MMDDhhmm[.ss]"))
+        specifies which frame to use for the cutoff comparison. \
+        The optional CC (century) defaults to the current century. \
+        The optional -hh (hours), mm (minutes) or ss (seconds) \
+        default to zero if not specified.",
+        metavar=("TENSE", "[CC]YYMMDD[-hh[mm[ss]]]"))
     p.add_argument("--silent", "--quiet", action="store_true",
         dest="silent", default=False,
         help="suppress errors and warnings")
@@ -1214,6 +1233,9 @@ def main() :
 
     args = p.parse_args()
 
+    # Grab environment variables if they exist and clean them
+    # up if they contain gargage..
+    #
     tmpExt = os.getenv("LSSEQ_IMAGE_EXTENSION")
     tmpOICExt = os.getenv("OIC_IMAGE_EXTENSION")
     if tmpExt != None and tmpExt != "" :
@@ -1266,6 +1288,11 @@ def main() :
     tmpExtList = sorted(tmpExtSet)
     CACHE_EXT = copy.deepcopy(tmpExtList)
 
+    #
+    # Respond to arguments set by user and/or set up variables
+    # etc. needed later based on the user's arguments.
+    #
+
     if args.printImgExtensions :
         print("Note:", PROG_NAME, "also recognizes all extensions below when uppercase.")
         extList = ":".join(IMAGE_EXT)
@@ -1296,6 +1323,9 @@ def main() :
             args.listWhichFiles = LIST_ONLYIMGS # Strictly only images.
 
     if args.cutoffTime != None :
+
+        # Do they want sequences 'before' or 'since' a given date?
+        #
         args.cutoffTime[0] = args.cutoffTime[0].lower()
         if (args.cutoffTime[0] != 'before') and (args.cutoffTime[0] != 'since') :
             if not args.silent :
@@ -1303,35 +1333,26 @@ def main() :
                     ": error: argument --onlyShow: TENSE must be 'since' or 'before'",
                     file=sys.stderr, sep='')
             sys.exit(1)
-        timeSpec = args.cutoffTime[1].split('.')
-        if len(timeSpec) <= 2 :
-            if   len(timeSpec[0]) == 12 :
-                timeFormat = "%Y%m%d%H%M"
-            elif len(timeSpec[0]) == 10 :
-                timeFormat = "%y%m%d%H%M"
-            elif len(timeSpec[0]) == 8 :
-                timeFormat = "%m%d%H%M"
-            else :
-                if not args.silent :
-                    print(PROG_NAME,
-                        ": error: argument --onlyShow: the time must be of the form [[CC]YY]MMDDhhmm[.ss]",
-                        file=sys.stderr, sep='')
-                sys.exit(1)
-        if len(timeSpec) == 2 :
-            timeFormat += ".%S"
 
-        try :
-            timeData=time.strptime(args.cutoffTime[1], timeFormat)
-        except ValueError :
+        # Process the cutoff time set.
+        #
+        tzName = datetime.datetime.now(datetime.timezone.utc).astimezone().tzname()
+        parsers = ['custom-formats']
+        timeData = dateparser.parse( \
+            args.cutoffTime[1], \
+            date_formats=DATE_FORMAT_LIST, \
+            settings={'TIMEZONE': tzName, 'PARSERS': parsers})
+
+        if timeData == None :
             if not args.silent :
                 print(PROG_NAME,
-                    ": error: argument --onlyShow: the time must be of the form [[CC]YY]MMDDhhmm[.ss]",
+                    ": error: argument --onlyShow: the time must be of the form [CC]YYMMDD[-hh[mm[ss]]]",
                     file=sys.stderr, sep='')
             sys.exit(1)
 
-        args.cutoffTime[1] = int(time.mktime(timeData)) # Epoch time
-
+        args.cutoffTime[1] = int(time.mktime(timeData.timetuple())) # Epoch time
 
+    # Now the meat and potatoes.
     # The following logic attempts to mimic the behavior
     # of /bin/ls as closely as possible.
 
